Package: DrugVsDisease
Type: Package
Title: Comparison of disease and drug profiles using Gene set
        Enrichment Analysis
Version: 2.55.0
Date: 2018-11-10
Author: C. Pacini
Maintainer: j. Saez-Rodriguez <saezrodriguez@ebi.ac.uk>
Description: This package generates ranked lists of differential gene
        expression for either disease or drug profiles. Input data can
        be downloaded from Array Express or GEO, or from local CEL
        files. Ranked lists of differential expression and associated
        p-values are calculated using Limma. Enrichment scores
        (Subramanian et al. PNAS 2005) are calculated to a reference
        set of default drug or disease profiles, or a set of custom
        data supplied by the user. Network visualisation of significant
        scores are output in Cytoscape format.
LazyData: yes
LazyLoad: yes
License: GPL-3
Depends: R (>= 2.10), affy, limma, biomaRt, ArrayExpress, GEOquery,
        DrugVsDiseasedata, cMap2data, qvalue
Imports: annotate, hgu133a.db, hgu133a2.db, hgu133plus2.db, RUnit,
        BiocGenerics, xtable
biocViews: Microarray, GeneExpression, Clustering
NeedsCompilation: no
Packaged: 2026-04-28 22:25:40 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/DrugVsDisease
git_branch: devel
git_last_commit: 04ae452
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
